"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addDefine = void 0;
const webpack_1 = require("webpack");
/*
  This edits the configuration of the WebpackPlugin
  It's called from the resolveForgeConfig hook.
*/
// copy-and-pasted from packages\api\core\src\util\plugin-interface.ts
function isForgePlugin(plugin) {
    return plugin.__isElectronForgePlugin;
}
function isWebpackConfiguration(mainConfig) {
    if (typeof mainConfig === "string")
        return false;
    if (typeof mainConfig === "function")
        return false;
    return true;
}
function addDefine(forgeConfig, defineKey, defineValue, log) {
    const webpackPlugin = forgeConfig.plugins.find((plugin) => plugin.name === "webpack");
    if (!webpackPlugin) {
        log("addDefine - WebpackPlugin is not configured");
        return false;
    }
    if (!isForgePlugin(webpackPlugin)) {
        log("addDefine - WebpackPlugin is not instantiated");
        return false;
    }
    const config = webpackPlugin.config;
    const mainConfig = config.mainConfig;
    if (!isWebpackConfiguration(mainConfig)) {
        log("addDefine - mainConfig is not instantiated");
        return false;
    }
    const definitions = {};
    // when the value is a string the Define plugin interprets it as code so it needs to be stringified
    definitions[defineKey] = JSON.stringify(defineValue);
    if (!mainConfig.plugins)
        mainConfig.plugins = [];
    mainConfig.plugins.push(new webpack_1.DefinePlugin(definitions));
    log(`addDefine - Added to Webpack configuration: ${JSON.stringify(definitions)}`);
    return true;
}
exports.addDefine = addDefine;
