"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourcePlugin = void 0;
const plugin_base_1 = require("@electron-forge/plugin-base");
const child_process_1 = require("child_process");
const fs = __importStar(require("fs/promises"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const EditWebpackConfig_1 = require("./EditWebpackConfig");
/*
  ResourcePlugin defines the API required of any plugin.
  Implementation defines the implementation of this plugin.
  ResourcePluginConfig declares the format of this plugin's configuration which the user defines in their package.json.
*/
function logif(b, message) {
    // note the instructions in the README about setting the DEBUG environment variable, otherwise this output is hidden
    if (b)
        console.log(`ResourcePlugin: ${message}`);
}
// determine which of the @electron-forge/cli/dist is being run
function isScript(scripts) {
    const cli = process.argv[1];
    return scripts.some((s) => cli.includes(`electron-forge-${s}`));
}
function isAlways(arg) {
    return arg.always !== undefined;
}
class Implementation {
    constructor(config) {
        var _a, _b, _c, _d, _e, _f;
        this.isPredefined = false;
        this.env = config.env;
        this.path = config.path;
        this.command = (_a = config.build) === null || _a === void 0 ? void 0 : _a.command;
        this.sources = (_b = config.build) === null || _b === void 0 ? void 0 : _b.sources;
        this.copydir = (_d = (_c = config === null || config === void 0 ? void 0 : config.package) === null || _c === void 0 ? void 0 : _c.copydir) !== null && _d !== void 0 ? _d : !!((_e = config.package) === null || _e === void 0 ? void 0 : _e.dirname);
        this.dirname = (_f = config === null || config === void 0 ? void 0 : config.package) === null || _f === void 0 ? void 0 : _f.dirname;
        this.verbose = config.verbose;
        if (this.dirname)
            this.tmpdir = path_1.default.join(os_1.default.tmpdir(), "electron-forge-resource-plugin", this.dirname);
        this.copied = this.copydir ? path_1.default.dirname(this.path) : this.path;
    }
    init(dir) {
        this.log(`init(${dir})`);
        this.dir = dir;
    }
    async resolveForgeConfig(forgeConfig) {
        this.log(`resolveForgeConfig`);
        // edit the forgeConfig to add the resource to packagerConfig.extraResource
        this.addPackagerExtraResource(forgeConfig);
        // edit the forgeConfig to add the define to packagerConfig.extraResource
        this.addWebpackDefine(forgeConfig);
        return Promise.resolve(forgeConfig);
    }
    async generateAssets(forgeConfig, platform, arch) {
        this.log(`generateAssets`);
        // build if needed
        if (await this.needsBuilding()) {
            if (!this.command) {
                throw new Error("Target `path` needs building but the `build.command` is not specified.");
            }
            await this.build(this.command);
            if (await this.needsBuilding()) {
                throw new Error("Target `path` still needs building after the `buildCommand` has been run.");
            }
        }
        // set the environment variable
        const environmentVariableValue = this.getEnvironmentVariableValue(true);
        this.setEnvironmentVariableValue(this.path);
    }
    async prePackage(forgeConfig, platform, arch) {
        this.log(`prePackage`);
        // generateAssets has already been called
        await this.copyToTmpdir();
        const environmentVariableValue = this.getEnvironmentVariableValue(false);
        this.setEnvironmentVariableValue(environmentVariableValue);
    }
    getEnvironmentVariableValue(isStart) {
        var _a;
        if (isStart)
            return this.path;
        // when it's packaged the filename specified by path will be in the resources directory or subdirectory
        var filename = path_1.default.basename(this.path);
        var dirname = (_a = this.dirname) !== null && _a !== void 0 ? _a : (this.copydir ? path_1.default.basename(path_1.default.dirname(this.path)) : undefined);
        return dirname ? path_1.default.join("resources", dirname, filename) : path_1.default.join("resources", filename);
    }
    addPackagerExtraResource(forgeConfig) {
        var _a;
        var resource = (_a = this.tmpdir) !== null && _a !== void 0 ? _a : this.copied;
        var extraResource = forgeConfig.packagerConfig.extraResource;
        if ((typeof extraResource === "string" && extraResource === resource) ||
            (Array.isArray(extraResource) && extraResource.some((element) => element === resource))) {
            this.log(`not setting extraResource again`);
            return;
        }
        if (!extraResource)
            extraResource = resource;
        else if (typeof extraResource === "string")
            extraResource = [extraResource, resource];
        else if (Array.isArray(extraResource))
            extraResource.push(resource);
        else
            throw new Error(`Unexpected extraResource value ${extraResource}`);
        this.log(`setting extraResource=${extraResource}`);
        forgeConfig.packagerConfig.extraResource = extraResource;
    }
    addWebpackDefine(forgeConfig) {
        // the path we want depends on whether it's being started locally or whether it's being packaged
        var environmentVariableValue;
        if (isScript(["make", "package", "publish"]))
            environmentVariableValue = this.getEnvironmentVariableValue(false);
        else if (isScript(["start"]))
            environmentVariableValue = this.getEnvironmentVariableValue(true);
        else {
            this.log("Unknown electron-forge script");
            return;
        }
        this.isPredefined = (0, EditWebpackConfig_1.addDefine)(forgeConfig, this.env, environmentVariableValue, this.log.bind(this));
    }
    setEnvironmentVariableValue(value) {
        // setting the environment variable is needed if the user must edit the Webpack Plugin configuration file,
        // but not if the addWebpackDefine method was able to edit the Webpack configuration programmatically
        if (this.isPredefined)
            return;
        // set the environment variable
        this.log(`setting 'process.env[${this.env}] = ${value};'`);
        process.env[this.env] = value;
    }
    async copyToTmpdir() {
        if (!this.tmpdir)
            return;
        var tmpdir = this.tmpdir;
        await fs.rm(tmpdir, { force: true, recursive: true });
        await fs.mkdir(tmpdir, { recursive: true });
        var destination = this.copydir ? tmpdir : path_1.default.join(tmpdir, path_1.default.basename(this.path));
        var opts = this.copydir ? { recursive: true } : undefined;
        this.log(`cp("${this.copied}", "${destination}")`);
        await fs.cp(this.copied, destination, opts);
    }
    async build(buildCommand) {
        this.log(`Build using: '${buildCommand}'`);
        const execAsync = (0, util_1.promisify)(child_process_1.exec);
        await execAsync(buildCommand);
        this.log(`Build complete.`);
    }
    async needsBuilding() {
        // if the file doesn't exist
        try {
            await fs.access(this.path);
        }
        catch {
            this.log(`Need to build because it does not exist: '${this.path}'`);
            return true;
        }
        // if it's older than sources
        if (this.sources) {
            let targetDate = (await fs.stat(this.path)).mtime;
            let sources = this.sources;
            // {always: boolean}
            if (isAlways(sources)) {
                let always = sources.always;
                if (always) {
                    this.log(`Need to build because 'sources' specifies '{always: true}'`);
                    return true;
                }
                return false;
            }
            // string | string[]
            let isNewer = (path, stats) => {
                if (stats.mtime > targetDate) {
                    this.log(`Need to build because source file is newer: '${path}'`);
                    return true;
                }
                return false;
            };
            let array = Array.isArray(sources) ? sources : [sources];
            for (let source of array) {
                this.log(`checking date of ${source}`);
                // ensure it exists
                try {
                    await fs.access(source);
                }
                catch {
                    throw new Error(`Source path not found: ${source}`);
                }
                // test whether isNewer
                const sourceStat = await fs.stat(source);
                if (sourceStat.isFile()) {
                    // it's a file
                    if (isNewer(source, sourceStat))
                        return true;
                }
                else if (sourceStat.isDirectory()) {
                    // else read the directory
                    for (const name of await fs.readdir(source)) {
                        const found = path_1.default.join(source, name);
                        try {
                            const foundStat = await fs.stat(found);
                            if (foundStat.isFile() && isNewer(found, foundStat))
                                return true;
                        }
                        catch {
                            this.log(`Cannot stat(${found})`);
                        }
                    }
                }
                else {
                    // else it's a link or maybe a device or something
                    throw new Error(`Source is neither a file nor a directory: ${source}`);
                }
            }
        }
        // else
        this.log(`Not rebuilding: '${this.path}'`);
        return false;
    }
    log(message) {
        logif(this.verbose, message);
    }
}
class ResourcePlugin extends plugin_base_1.PluginBase {
    constructor(c) {
        logif(c.verbose, "constructor");
        super(c);
        this.name = "resource";
        this.impl = new Implementation(c);
        this.init = this.init.bind(this);
        this.getHooks = this.getHooks.bind(this);
    }
    init(dir, config) {
        this.impl.init(dir);
        super.init(dir, config);
    }
    getHooks() {
        this.impl.log(`getHooks()`);
        // see https://www.electronforge.io/configuration#hooks for a list of the hook names
        // the parameters passed to different hook functions can be seen by searching electron forge source for "runHook"
        return {
            resolveForgeConfig: this.impl.resolveForgeConfig.bind(this.impl),
            generateAssets: this.impl.generateAssets.bind(this.impl),
            prePackage: this.impl.prePackage.bind(this.impl),
        };
    }
}
exports.default = ResourcePlugin;
exports.ResourcePlugin = ResourcePlugin;
